/*********************************************************************/
/*                                                                   */
/*   Program Name :  aux_recov                                       */
/*                                                                   */
/*      Version:  1.00  2008-02-19                                   */
/*                                                                   */
/*                                                                   */
/*      AUX field recover for K5/VSSP32 data                         */
/*                                                                   */
/*                                                                   */
/*      Copyright (c) 2007-2008 T.Kondo/NICT All Right Reserved      */
/*                                                                   */
/*********************************************************************/
/*
      usage
             aux_recov file_name [ofile]
                  where   file_name -- K5/VSSP32 data file name
                          ofile      - outfile name (default datafile.recov)

        Ver 0.00 2007-03-15  start development
        Ver 1.00 2008-02-19  option '--help' and '--ver[sion]' were added
*/
#define PROGRAM_VERSION "Ver. 2008-02-19"

#include "libk5.h"      // common header for IP-VLBI utilities

//#ifndef WIN32
//#include <stdint.h>
//#endif
//#define REWIND_BYTE  0  // define how many bytes rewind if failure in 
                          // header detection  on 1st trial (shoud be multiples of 4)
                          // 0 means rewind till previous header
#define REWIND_BYTE 200 

#ifdef WIN32
#define USE_ASM    // use inline assemble function
#endif

#define SYNC_SEARCH_BYTE_STEP  // Sync pattern search step
//#define SYNC_SEARCH_2BYTES_STEP 
//#define SYNC_SEARCH_4BYTES_STEP 

#include "libvssp32.h"

typedef struct DATACHK_INTERNAL_USE  // copy from datachk.c (for datachk internal use)
{
    char fname[128], keepfname[128];
	int nodatfile;  // data file not found flag  1: not found 0: found
	int mode; // processing mode
    int bitslip_by_frame[3600];
	int total_frame;
	int err_all;  // # of frame where any error occurred
	//int err_frame;  // # of frame where data are substituted by artifical data (VSSP32 only)
	int eflg_frame;  // # of frame where data are substituted by artifical data (VSSP32 only)
	//int bad_frame;  // # of bit slip or bit make occurred frame
	int slip_frame;  // # of bit slip or bit make occurred frame
	int aux_err;     // # of aux field error frame
	int aux_misalign;     // # of aux field misalign frame
	int bit_rev_err;     // # of 1 bit reversal occurred in sync field
	int time_discon;   //# of time discontinuity  (not bit slip and make)
	int time_discon_bs;  // # of time discon and bit slip or make
	int flg_1st64_ok;   // flag  1: 1st 64bit data is header  0: no 
	int byteoffset_1st_header;   // # of byte offset to detect 1st header
	int sfreq;   //sampling frequency (in kHz)
	int adbit;   // AD bit length
	int numch;   // # of CH
    int filter;  // LPF filter frequency(MHz) for K5/VSSP32 (0: through)
    int hh, mm, ss;
    int seconds; // second from 0h UTC
    int hhe, mme, sse;
    int seconds_end; // second from 0h UTC
    // following is valid for K5/VSSP32
    int year;    // 3 digit year
    int totalday;    // total day
    int yeare;    // 3 digit year
    int totaldaye;    // total day
	//
	int k5format;  // 1: K5/VSSP    2: K5/VSSP32
	int keepstatus; // 0: nothing to do  1: keep renamed  2: keep copied 
	int64_t size;  // opened file size in bytes **K5**
	int64_t tbytes;   // total # of bytes read **K5**
    char fmt[10];
} DATACHK_INTERNAL_USE_T;


void dt_intuse_ini(DATACHK_INTERNAL_USE_T *dt)
{
/*
   initialize DATACHK_INTERNAL_USE_T type variable

  */
	int n;
	dt->fname[0]='\0';
	dt->keepfname[0]='\0';
	dt->keepstatus=0;
	for(n=0; n<3600; n++) dt->bitslip_by_frame[n]=0;
	dt->total_frame=0;
	dt->err_all=0;
	dt->eflg_frame=0;
	dt->slip_frame=0;
	dt->aux_err=0;
	dt->aux_misalign=0;
	dt->bit_rev_err=0;
	dt->time_discon=0;
	dt->time_discon_bs=0;
	dt->byteoffset_1st_header=0;
	dt->sfreq=0;
	dt->adbit=0;
	dt->numch=0;
	dt->filter=0;
	dt->hh=0;
	dt->mm=0;
	dt->ss=0;
	dt->seconds=0;
	dt->hhe=0;
	dt->mme=0;
	dt->sse=0;
	dt->seconds_end=0;
	dt->year=0;
	dt->totalday=0;
	dt->yeare=0;
	dt->totaldaye=0;
	dt->fmt[0]='\0';
}

// K5 header check (updated from headerchkn2)
int K5header_parse(K5VSSP_file_t *K5)
/*
    K5 header common portion parse function
	only header portion stored in K5.head32[8] is analyzed.

    allow at least 1 bit error in sync pattern

    return 0 : K5/VSSP Header detected
          -1 : K5.head32 is not header

*/
{
    int sdat[16]={1,2,4,8,0,0,0,0,0,0,0,0,0,0,0,0};
    int fdat[16]={40,100,200,500,1000,2000,4000,8000,16000, 
                  32000,64000,128000,256000,512000,1024000,2048000}; 
    int smplbit0, sfreq0, nch;
    u_int32_t syncpat=0x8B000000UL;    // 2nd sync pattern for K5/VSSP
    u_int32_t syncpat32=0x8C000000UL;  // 2nd sync pattern for K5/VSSP32
	u_int32_t w1, w2; //2007-03-02
	int n, nbit1, nbit2; // 2007-03-02
    int iok, iok2;
    iok=-1;

    K5->format=0;  // suppose K5/VSSP old format

	iok2=0;
	// First 32bit sync pattern detection
    if(K5->head32[0]==0xFFFFFFFFUL) iok2=1;

	nbit1=0;
    nbit2=0;

#ifdef ALLOW_1BIT_REVERSE

    // allow in case of only one bit difference in FF field
	if(iok2==0){
       w1= K5->head32[0] ^ 0xFFFFFFFFUL;
	   for(n=0; n<32; n++){
	      if((w1 & 0x1UL)== 1) nbit1++;
		  w1>>= 1;
	   }
	   if(nbit1 <= 1) iok2=1;
	}
#endif
	// 2nd sync pattern detection
	if(iok2 == 1){
        if((K5->head32[1] & 0xFF000000UL) == syncpat) K5->format=1; // this is sync block
        if((K5->head32[1] & 0xFF000000UL) == syncpat32) K5->format=2; // this is sync block
        // allow in case of only one bit difference in 2nd sync field
#ifdef ALLOW_1BIT_REVERSE

	    if(K5->format==0){       // 2007-03-02
            w2 = K5->head32[1] ^ syncpat;
			w2 >>= 24;
		    for(n=0; n<8; n++){
			   if((w2 & 0x1UL)== 1) nbit2++;
			   w2>>= 1;
			}
			if(nbit2 <= 1){
				K5->format=1;
			} else {
			   nbit2=0;
               w2 = K5->head32[1] ^ syncpat32;
			   w2 >>= 24;
		       for(n=0; n<8; n++){
			      if((w2 & 0x1UL)== 1) nbit2++;
			      w2>>= 1;
			   }
			   if(nbit2 <= 1) K5->format=2;
			}
		}
#endif

	}

	if(K5->format > 0){ 
            iok=0;
            /* parse sampling information and get time data */ 
            K5->spatn=(int)((K5->head32[1] >> 24) & 0xFF);       
            smplbit0 = (K5->head32[1] >> 22) & 0x3;   
            K5->adbit = sdat[smplbit0];      // AD bits
            sfreq0 = ( K5->head32[1] >> 18) & 0xF;
            K5->sfreq=fdat[sfreq0];
            nch = (K5->head32[1] >> 17) & 1;
            if (nch == 1) {
                K5->numch = 4; 
            } else {
                K5->numch = 1;
            }
            K5->seconds=(long int)(K5->head32[1] & 0x1FFFFUL);
            sec2hms(K5->seconds,&K5->hh,&K5->mm,&K5->ss);
			if(K5->framenum==0){
				K5->sfreq1st=K5->sfreq;
				K5->adbit1st=K5->adbit;
				K5->numch1st=K5->numch;
				K5->seconds1st=K5->seconds;
				K5->secold=K5->seconds;
			}
			K5->framenum++;
	}

    return iok;
}

int aux_err_chk(DATACHK_INTERNAL_USE_T *dt) 
{
    /* check AUX field error check only 
	   read only 2 frames 

	  return     0:  no error
	            -1:  no data file
	            -2:  recoverable AUX error
			   -99:  unrecoverable error
	*/

       int usampl=200000;
       //int i,n,kword, kbytes;
       int i,n,kbytes;
       int64_t tword, tbytes;
       long int numb,npos, tflag, nkai;
       long int bitsec,bitint;
       int headerbits;
       int f_c;
       int total_frame, slip_frame, time_discon,time_discon_bs,aux_err;
	   int bit_rev_err, aux_misalign;
	   int err_all;
	   int eflg_frame; // statistics for VSSP32 only
	   char fname[120];
       char fmt[10];
	   char lbuf[120];
       double dbytes;
       K5VSSP_file_t K5;
       vsspaux_t aux;
       //fpos_t size;

       k5vssp_t_ini(&K5);
       total_frame=0;
	   eflg_frame=0; // statistics for VSSP32 error flag only
       slip_frame=0;
	   aux_err=0;
	   aux_misalign=0;
	   bit_rev_err=0;
	   time_discon=0;
	   time_discon_bs=0;
       numb=0;
       npos=0;
       nkai=0;

       tflag=0;
       f_c=0;
       dbytes=0.0;

       headerbits=64;  // for K5/VSSP
	   lbuf[0]='\0';

       strcpy(fname,dt->fname);
       strcpy(K5.fname,fname);

	   dt->flg_1st64_ok=1;   // flag default: 1st 64bits of file is headder  
	   dt->filter=-1;


       /* K5/VSSP32 file open and 1st header read 
               to get sampling information etc */

	   dt->nodatfile=0;
       K5.size=get_file_size(K5.fname);
	   if(K5.size == -1){
           printf("File %s not found! \n",K5.fname);
	       dt->nodatfile=1;
           goto exitend2;
	   }
	   dt->size=K5.size;
       if( (K5.stream  = fopen( K5.fname, "rb" )) == NULL ) {
           //printf("File %s Open Error! \n",K5.fname);
           printf("File %s not found! \n",K5.fname);
           goto exitend2;
       }
       //printf("FILE : %s  (%lld bytes)\n",K5.fname,K5.size);
       printf("FILE : %s  (%u bytes)\n",K5.fname,K5.size);

       tword=0;
	   tbytes=0;

       //if(checkheader(&K5) < 0){
       if((kbytes=CheckK5header_with_Read(&K5)) < 0){
            printf("File read error! \n");
            //fgetpos(K5.stream,&K5.pos); 
            i=fclose( K5.stream ) ;
            goto exitend2;
       }

	   dt->k5format=K5.format;

       if(K5.format==2) headerbits=96+K5.auxbyte*8;  // usually 256
       //printf("headerbits= %d\n",headerbits);

       tbytes+=K5.tbytesread;
       K5.twordread=0;
       K5.tbytesread=0;

       K5.fsample=1000.0*K5.sfreq;   // sampling frequency (Hz)
       K5.tsample=1.0/K5.fsample;            // sampling period (sec)


       bitsec=(long)K5.fsample*K5.adbit*K5.numch;  // # of bits in 1 sec data except header
       K5.bytesec=bitsec/8;                  //# of bytes in 1 sec data
       K5.imax=(K5.sfreq*1000)/usampl;        // # of usampls in a second
       K5.numb=usampl*K5.adbit*K5.numch/8;       // # of bytes in a usampl
       K5.nword=K5.numb/4;                 // # of 32bit-word in a usampl

       K5.unitp=(double)usampl*K5.tsample;   // period in a usampl (sec)

       //K5.ibuf  = (unsigned char *)malloc(K5.numb); // K5 data read buffer
       K5.ubuf  = (u_int32_t *)malloc(K5.nword*sizeof(u_int32_t)); // K5 data read buffer

	   dt->byteoffset_1st_header=kbytes;
       if(kbytes >0){
           printf("First 64bit is not a header block!\n");
		   dt->flg_1st64_ok=0;
       }

       strcpy(fmt,"VSSP");
       if(K5.format==2) strcpy(fmt,"VS32");

	   strcpy(dt->fmt,fmt);

	   /* keep first frame info */
	   dt->adbit=K5.adbit;
	   dt->numch=K5.numch;
	   dt->sfreq=K5.sfreq;
	   dt->hh=K5.hh;
	   dt->mm=K5.mm;
	   dt->ss=K5.ss;
	   dt->seconds=K5.seconds;
       if(K5.format==2){
	      dt->year=K5.year;
	      dt->totalday=K5.totalday;
	   }

       if(K5.format==2){
           printf("\n This is K5/VSSP32 format data.\n");
           printf("\n  1st Extended Header Info is as follows.\n");
           printf("    Year = %d    Total Day = %d\n", K5.year,K5.totalday);
           printf("    Version  Major = %d  Minor = %d    # of AUX Bytes = %d\n",
               K5.version1,K5.version2,K5.auxbyte);
           printf("    AUX Field Data : ");
           for(n=0; n<K5.auxbyte/4; n++){
               printf(" %04X", K5.aux32[n]);
           }
           aux.size=K5.auxbyte;
           //strncpy(aux.bdat,(char *)K5.aux32,K5.auxbyte); // this is not good for in case of data 0 is included
           for(n=0; n<aux.size; n++) aux.bdat[n]=*((char *)K5.aux32+n);
           vssp_aux_parse(&aux);
           printf("\n        Format # = %d\n",aux.fmt);
           if(aux.fmt != 0){
             printf("        Filter (MHz) : %d\n",aux.filter);
			 dt->filter=aux.filter;
		   }
           if(aux.fmt == 1)
             printf("        Station info : ID2(%s) name(%s)\n",aux.sid2,aux.station8);
           if(aux.fmt == 1 || aux.fmt == 2 )
             printf("        PC host name :  %s\n",aux.hostname8);
           printf("\n");
       }

       if((kbytes=fwd1secK5(&K5))<0){
           tbytes+=K5.tbytesread;
           //printf("File EOF  %d\n", K5.tbytesread);
           goto exitend;
       }
       bitint=(int)(K5.tbytesread*8-headerbits);  //64bit is headder
       tbytes+=K5.tbytesread;
       K5.tbytesread=0;  // reset read counter
       total_frame++;

	   if(K5.ierr < 0){
		   if(abs(K5.ierr)/10 == 1) bit_rev_err++;
		   if(abs(K5.ierr)%10 == 2) aux_err++;
		   if(abs(K5.ierr)%10 == 1) aux_misalign++;
	   } else {
		   if(K5.eflg == 1) eflg_frame++;  // because eflg is in AUX field
	   }
       if (bitint != bitsec){
	      dt->bitslip_by_frame[slip_frame]=bitsec-bitint;
          slip_frame++;
		  if(tflag !=0) time_discon_bs++;
       } else {
		  if(tflag !=0) time_discon++;
	   }

exitend:
       i=fclose( K5.stream ) ;
       free(K5.ubuf);
exitend2:
	   if(dt->nodatfile == 1) return -1;

	   dt->slip_frame=slip_frame;
	   dt->total_frame=total_frame;
	   dt->time_discon=time_discon;
	   dt->time_discon_bs=time_discon_bs;
	   dt->aux_err=aux_err;
	   dt->aux_misalign=aux_misalign;
	   dt->bit_rev_err=bit_rev_err;
	   dt->eflg_frame=eflg_frame;
	   //err_all=slip_frame+time_discon+time_discon_bs+aux_err+eflg_frame;
	   err_all=slip_frame+time_discon+time_discon_bs+aux_err+eflg_frame+bit_rev_err;
	   if(slip_frame == 1 && aux_err == 0 && eflg_frame == 0 
		   && time_discon == 0 && aux_misalign == 1){
		   // recoverable AUX field error
		   //printf("This is recoverable AUX error!\n");
		   //printf("Bit error  %d\n",dt->bitslip_by_frame[0]);
		   return -2;
	   }
	   if(err_all > 0) return -99;
	   return 0;
}

void aux_field_recov(char* fname, char* oname, int slipbyte)
{
	   int usampl=200000;
       int i,n;
       int64_t tword, tbytes;
	   u_int32_t head32[8];
       unsigned char *head8;
       long int numb,npos,tflag, nkai;
       long int bitsec;
	   int kbytes;
       int nwrite;
       //int rword;
       int f_c;
       int xhist[8];     // 8=4ch*2  (" means sign+  sign- of data)
       char cbuf[10];
       char fmt[10];
       int total_frame, bad_frame;
       //int wordread;
       int headerbits;
       double dbytes;
       u_int32_t syncpat=0x8B000000UL;    // 2nd sync pattern for K5/VSSP
       u_int32_t syncpat32=0x8C000000UL;  // 2nd sync pattern for K5/VSSP32
       K5VSSP_file_t K5;
       vssp_head_t vsph;
       FILE *fo;

       k5vssp_t_ini(&K5);
       total_frame=0;

       K5.aux32=K5.head32+3;

	   K5.head8=(unsigned char*)K5.head32;
	   head8=(unsigned char*)head32;

       total_frame=0;
       bad_frame=0;

       numb=0;
       npos=0;
       nkai=0;

       tflag=0;
       f_c=0;
       dbytes=0.0;
       for(i=0; i<8; i++) xhist[i]=0; 


       /* K5/VSSP file open and 1st header read 
               to get sampling information etc */

       strcpy(K5.fname,fname);

       K5.size=get_file_size(K5.fname);
       if( (K5.stream  = fopen( fname, "rb" )) == NULL ) {
           printf("File %s Open Error! \n",K5.fname);
           goto exitend2;
       }
       printf("FILE : %s  (%lld bytes)\n",K5.fname,K5.size);

       K5.twordread=0;  // reset word read counter
       K5.pos=0;        // reset current pointer position
       tword=0;
	   tbytes=0;

       //if(checkheader(&K5) < 0){
       if((kbytes=CheckK5header_with_Read(&K5)) < 0){
            printf("File read error! \n");
            //fgetpos(K5.stream,&K5.pos); 
            i=fclose( K5.stream ) ;
            goto exitend2;
       }
       // out file open

       if( (fo  = fopen( oname, "wb" )) == NULL ) {
           printf("File %s Open Error! \n",oname);
           goto exitend2;
       }

       if(K5.format==2) headerbits=96+K5.auxbyte*8;  // usually 256
       //printf("headerbits= %d\n",headerbits);

       dbytes=kbytes*1.0+K5.headsizew*4.0;
       //printf("kword, headsizew, dbytes %d %d %f\n",kword,K5.headsizew,dbytes);

       tbytes+=K5.tbytesread;
       K5.twordread=0;

       K5.fsample=1000.0*K5.sfreq;   // sampling frequency (Hz)
       K5.tsample=1.0/K5.fsample;            // sampling period (sec)


       bitsec=(long)K5.fsample*K5.adbit*K5.numch;  // # of bits in 1 sec data except header
       K5.bytesec=bitsec/8;                  //# of bytes in 1 sec data
       K5.imax=(K5.sfreq*1000)/usampl;        // # of usampls in a second
       K5.numb=usampl*K5.adbit*K5.numch/8;       // # of bytes in a usampl
       K5.nword=K5.numb/4;                 // # of 32bit-word in a usampl

       K5.unitp=(double)usampl*K5.tsample;   // period in a usampl (sec)

       //K5.ibuf  = (unsigned char *)malloc(K5.numb); // K5 data read buffer
       K5.ubuf  = (u_int32_t *)malloc(K5.nword*sizeof(u_int32_t)); // K5 data read buffer

       if(kbytes >0){
           printf("First 64bit is not a header block!\n");
       }

       strcpy(fmt,"VSSP");
       if(K5.format==2) strcpy(fmt,"VS32");

	   /* keep first frame info */
	   vsph.adbit=K5.adbit;
	   vsph.numch=K5.numch;
	   vsph.sfreq=K5.sfreq;
	   vsph.hh=K5.hh;
	   vsph.mm=K5.mm;
	   vsph.ss=K5.ss;
	   vsph.seconds=K5.seconds;
       if(K5.format==2){
	      vsph.year=K5.year;
	      vsph.totalday=K5.totalday;
	   }

	   /* save 1st header */
	   memcpy(head32, K5.head32, 32);

       if(K5.format==2){
           printf("\n This is K5/VSSP32 format data.\n");
           printf("\n  1st Extended Header Info is as follows.\n");
           printf("    Year = %d    Total Day = %d\n", K5.year,K5.totalday);
           printf("    Version  Major = %d  Minor = %d    # of AUX Bytes = %d\n",
               K5.version1,K5.version2,K5.auxbyte);
           printf("    AUX Field Data : ");
           for(n=0; n<K5.auxbyte/4; n++){
               printf(" %0X", K5.aux32[n]);
           }
           printf("\n\n");
       }


       printf(" FMT A/D CH  f(kHz)  TIME   seconds      AUX FIELD\n");
       printf("%s %2d  %2d  %5d %02d:%02d:%02d %6d  ", fmt,
                  K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);
	   for(i=0; i<2; i++) printf(" %08X",K5.head32[i+2]); printf("  .... \n"); 

       // copy 1st header data 
       nwrite = fwrite( head32, sizeof(u_int32_t), 8, fo);

	   // copy 1st frame data
       for(n=0; n<K5.imax; n++){
           K5.wordread = fread(K5.ubuf, sizeof(u_int32_t), K5.nword, K5.stream);
             if( ferror( K5.stream )  ) {
                 //printf("File Read Error!\n");
                 break;
			 }
             nwrite = fwrite( K5.ubuf, sizeof(u_int32_t), K5.wordread, fo);
	   }
	   // skip slipbyte bytes
       kbytes = fread(cbuf, sizeof(char), -slipbyte, K5.stream);

       while(1){
           // read header
           K5.wordread = fread(K5.head32, sizeof(u_int32_t), 8, K5.stream);
           if( ferror( K5.stream )  || K5.wordread == 0)  break;
           K5header_parse(&K5);  // to obtain sampling parameter & time 
		   // header recover according to the rule
		   //  9 and 10 th byte data moved to 31 and 32nd bytes
		   memcpy(head8, K5.head8, 8);
		   memcpy(head8+8, K5.head8+30, 2);
		   memcpy(head8+10, K5.head8+8, 22);
           // write recovered header data 
           nwrite = fwrite( head32, sizeof(u_int32_t), 8, fo);
       
		   printf("%s %2d  %2d  %5d %02d:%02d:%02d %6d  ", fmt,
                  K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);
	       for(i=0; i<2; i++) printf(" %08X",head32[i+2]); printf("  .... \n"); 

	       // copy 1sec data
           for(n=0; n<K5.imax; n++){
               K5.wordread = fread(K5.ubuf, sizeof(u_int32_t), K5.nword, K5.stream);
               if( ferror( K5.stream )   || K5.wordread == 0 ) {
                  //printf("File Read Error!\n");
                  break;
			   }
               nwrite = fwrite( K5.ubuf, sizeof(u_int32_t), K5.wordread, fo);
		   }

	   }
       i=fclose( K5.stream ) ;

 exitend2:
        i=fclose( fo ) ;
        free(K5.ubuf);
        printf("\n");
}


int main(int argc, char *argv[])
{
    char oname[60];
    char* dflt_fname="tds.data";
	int i;
	int slipbyte;
    int mode;
    double telapsed;           // to measure processing time
    clock_t cstart1sec, cfinish1sec;  // to calculate processing time
    DATACHK_INTERNAL_USE_T dt;
    mode=0;
    if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
       //strcpy(fname,dflt_fname);
       printf("aux_recov %s\n",PROGRAM_VERSION);
       printf("How to run: \n");
       printf("  aux_recov datafile [ofile]\n");
       printf("\n");
       printf("    where  datafile ----- K5/VSSP32 data file name to recover\n");
       //printf("                          if 0 is given, internal dflt name will be used\n");
       printf("           ofile      --- outfile name (default datafile.recov)\n");
       exit(0);
    }
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
       printf("aux_recov %s\n",PROGRAM_VERSION);
       exit(0);
    }
    // initialize dt
    dt_intuse_ini(&dt);

    if (argc>=2){
       strcpy(dt.fname,*(argv+1));
    }
    oname[0]='\0';
    if(argc >=3){
       strcpy(oname,*(argv+2));
    }
    // oname creation
    if(oname[0]=='\0'){
        strcpy(oname,dt.fname);
        strcat(oname,".recov");
    }
    cstart1sec = clock();
    printf("************ aux_recov  %s ************\n",PROGRAM_VERSION);
    //printf("  format conversion start!\n");
    i=aux_err_chk(&dt);
	if(i==0){
		printf("AUX data of 2nd frame seems to be OK.\n So nothing to do.\n");
		return 0;
	}
	if(i== -2){
		printf("  Recoverrable AUX field error was detected!\n");
		slipbyte=dt.bitslip_by_frame[0]/8;  // 
		printf("  Recover condition : (slip byte = %d)\n",slipbyte);
		printf("\n");
		printf("AUX FIELD recover start!!\n\n");
        aux_field_recov(dt.fname,oname,slipbyte);
		printf("Original  File : %s\n",dt.fname); 
		printf("Recovered File : %s\n",oname); 
	}
    cfinish1sec = clock();
    telapsed = (double)(cfinish1sec - cstart1sec) / CLOCKS_PER_SEC;
    printf("Time elapsed for processing is %f sec\n",telapsed);
    return 0;
}
