#!/bin/sh
#

if [ -z $1 ]; then   #  display how to use
echo '

# K5/VSSP32 check procedure by T.Kondo 2007-02-16
#
#  How to execute 
#     
#   ./vssp32test.sh span [nkai [sfmax [data_folder 
#         [datachk_log_file [sampling_log_file
#         [error_log_file ]]]]]
#       where 
#          span  --------  sampling period for a scan (sec)
#          nkai     ----- # of repeat   (default is 1)
#          sfmax    ----- maximum sampling frequency 2,4,8,16,32,64
#                           (default is 64)
#          data_folder -- folder where sampled data file is created
#                          (default is current directory)
#          datachk_log_file -- file name for 'datachk' summary output
#                               (default is "v32dtchkYYMMDDHHMMSS.HOST.log")
#          sampling_log_file ---- log file name for 'sampling' program
#                               (default is  "v32smplgYYMMDDHHMMSS.HOST.log")
#          error_log_file  ---- error log file name for 'datachk' program
#                               (default is  "v32errorYYMMDDHHMMSS.HOST.log")
#              where YYMMDDHHMMSS is start time of this shell execution
#                    HOST is a host PC name
#
#   Check items    
#         sampling frequency :  2, 4, 8, 16, 32, 64 MHz
#         AD bits            :  1
#         Channels           :  1, 4
#         Scan length        :  given by operator 
#         # of repeat        :  1 or given by operator 
#
#   Checking above all items (frequency max is sfmax it is given by
#   an operator) will be repeated 'nkai' times
#
#   Typical execution example
#      ./vssp32test.sh 300 24
#         ==>  this will give 24 hour check of 300 sec scans.

'
else

# actual execution
pdir="./"        # program directory
spl="sampling"
chk="datachk"
ana="vssplogana"
dco="setdcoffset"
ad=1
count=0
tscan=0
span=$1     # sampling span (sec)
nkai=$2     # repeat number
sfmax=$3    # sampling frequency max
d_dir=$4    # data out folder
d_log=$5    # log file name for 'datachk'
s_log=$6    # log file name for 'sampling' and 'setdcoffset'
e_log=$7    # errpr log file name for 'datachk'

# get system time as YYMMDDHHMMSS
ymd=`date +%y%m%d%H%M%S`
# get a host PC name
hn=`uname -n`

if [ $1 = 0 ]; then  # $1 is 0
   span=300
fi
if [ -z $2 ]; then  # $2 is null
   nkai=1
fi
if [ -z $3 ]; then  # $3 is null
   sfmax=64
fi
if [ -z $4 ]; then  # $4 is null
   d_dir="./"
fi
if [ -z $5 ]; then  # $5 is null
   d_log="v32dtchk"$ymd".$hn.log"
fi
if [ -z $6 ]; then  # $6 is null
   s_log="v32smplg"$ymd".$hn.log"
fi
if [ -z $7 ]; then  # $7 is null
   e_log="v32error"$ymd".$hn.log"
fi
echo 'span  = ' $span
echo 'nkai  = ' $nkai
echo 'sfmax = ' $sfmax
echo 'data directory    is ' $d_dir
echo 'datachk  log file is ' $d_log
echo 'sampling and setdcoffset log file is ' $s_log
echo 'datachk error log file is ' $e_log

# delete sampling log file just in case
# echo 'delete existed sampling log file (' $s_log ')'
# rm $s_log

# set board time using PC time to synchronize time in log file
./timesetpc 1

# set DC affset adequate for testing
echo $pdir$dco TEST $s_log
$pdir$dco TEST $s_log

# loop counter starts from 1
count=`expr $count + 1`

# repeating kaisu
while [ $count -le $nkai ];
do 

   # Loop in sampling frequency
   sf=2
   # for sf in 2 4 8 16 32 64
   while [ $sf -le $sfmax ];
   do
       #  Loop in Channel
       for ch in 1 4
       do
         echo '******** Loop# ' $count 'out of ' $nkai '********'
         echo $sf'MHz '$ad'bit '$ch'ch '$span'sec testing...'
         echo $pdir$spl -$span $sf $ad $ch $d_dir/test$ch.dat $s_log
         $pdir$spl -$span $sf $ad $ch $d_dir/test$ch.dat $s_log
         if [ $tscan -eq 0 ]; then
            # datacheck log file newly created
            echo     $pdir$chk $d_dir/test$ch.dat 0 $d_log $e_log 1
            $pdir$chk $d_dir/test$ch.dat 0 $d_log $e_log 1
         else
            # datacheck log file open as append mode
            echo     $pdir$chk $d_dir/test$ch.dat 0 -$d_log -$e_log 1
            $pdir$chk $d_dir/test$ch.dat 0 -$d_log -$e_log 1
         fi
         # log file analysis
         echo $pdir$ana $s_log $d_log
         $pdir$ana $s_log $d_log
         tscan=`expr $tscan + 1`
       done
       sf=`expr $sf \* 2`    #  sf=sf*2
   done
   count=`expr $count + 1`
done

# reset DC affset adequate for testing
echo $pdir$dco RESET $s_log
$pdir$dco RESET $s_log

# end of actual execution
echo ' '
echo 'VSSP32 TEST completed.'
echo '   sampling log file        : '$s_log
echo '   datachk summary log file : '$d_log
echo '   datachk error   log file : '$e_log
echo '  You can execute '$ana' again to analyze above log files as follows.'
echo '   '$pdir$ana $s_log $d_log
echo '  See '$e_log' for kept errored data files.'
echo ' '
fi
# end of all shell script
